@@ -17,6 +17,9 @@
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
+#ifndef NUT_CGILIB_H_SEEN
+#define NUT_CGILIB_H_SEEN 1
+
 #ifdef __cplusplus
 /* *INDENT-OFF* */
 extern "C" {
@@ -41,3 +44,4 @@ int checkhost(const char *host, char **desc);
 /* *INDENT-ON* */
 #endif
 
+#endif	/* NUT_CGILIB_H_SEEN */
