@@ -21,7 +21,7 @@
 #include "serial.h"
 
 #define DRIVER_NAME	"Ever UPS driver (serial)"
-#define DRIVER_VERSION	"0.07"
+#define DRIVER_VERSION	"0.08"
 
 /* driver description structure */
 upsdrv_info_t upsdrv_info = {
@@ -177,7 +177,9 @@ void upsdrv_updateinfo(void)
 static
 int instcmd(const char *cmdname, const char *extra)
 {
+	/* May be used in logging below, but not as a command argument */
 	NUT_UNUSED_VARIABLE(extra);
+	upsdebug_INSTCMD_STARTING(cmdname, extra);
 
 	/* FIXME: Which one is this - "load.off",
 	 * "shutdown.stayoff" or "shutdown.return"? */
@@ -185,14 +187,15 @@ int instcmd(const char *cmdname, const char *extra)
 	/* Shutdown UPS */
 	if (!strcasecmp(cmdname, "load.off"))
 	{
+		upslog_INSTCMD_POWERSTATE_CHANGE(cmdname, extra);
 		if (!Code(2)) {
-			upslog_with_errno(LOG_INFO, "Code failed");
+			upslog_with_errno(LOG_INSTCMD_UNKNOWN, "Code failed");
 			return STAT_INSTCMD_UNKNOWN;
 		}
 		ser_send_char(upsfd, 28);
 		ser_send_char(upsfd, 1);  /* 1.28 sec */
 		if (!Code(1)) {
-			upslog_with_errno(LOG_INFO, "Code failed");
+			upslog_with_errno(LOG_INSTCMD_UNKNOWN, "Code failed");
 			return STAT_INSTCMD_UNKNOWN;
 		}
 		ser_send_char(upsfd, 13);
@@ -201,7 +204,7 @@ int instcmd(const char *cmdname, const char *extra)
 		return STAT_INSTCMD_HANDLED;
 	}
 
-	upslogx(LOG_NOTICE, "instcmd: unknown command [%s] [%s]", cmdname, extra);
+	upslog_INSTCMD_UNKNOWN(cmdname, extra);
 	return STAT_INSTCMD_UNKNOWN;
 }
 
