@@ -1,4 +1,4 @@
-/* nutdrv_qx_blazer-common.c - Common functions/settings for nutdrv_qx_{innovart31,mecer,megatec,megatec-old,mustek,q1,q2,q6,voltronic-qs,zinto}.{c,h}
+/* nutdrv_qx_blazer-common.c - Common functions/settings for nutdrv_qx_{innovart31,innovart33,mecer,megatec,megatec-old,mustek,q1,q2,q6,voltronic-qs,zinto}.{c,h}
  *
  * Copyright (C)
  *   2013 Daniele Pezzini <hyouko@gmail.com>
@@ -191,6 +191,8 @@ void	blazer_initups_light(item_t *qx2nut)
 /* Preprocess setvars */
 int	blazer_process_setvar(item_t *item, char *value, const size_t valuelen)
 {
+	/* upsdebug_SET_STARTING(item->info_type, value); */
+
 	if (!strlen(value)) {
 		upsdebugx(2, "%s: value not given for %s", __func__, item->info_type);
 		return -1;
@@ -231,7 +233,8 @@ int	blazer_process_setvar(item_t *item, char *value, const size_t valuelen)
 
 	} else {
 
-		/* Don't know what happened */
+		/* Don't know what happened: unknown entry for pre-processing? */
+		/* upslog_SET_UNKNOWN(item->info_type, value); */
 		return -1;
 
 	}
@@ -242,15 +245,7 @@ int	blazer_process_setvar(item_t *item, char *value, const size_t valuelen)
 /* Preprocess instant commands */
 int	blazer_process_command(item_t *item, char *value, const size_t valuelen)
 {
-#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
-#pragma GCC diagnostic push
-#endif
-#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
-#pragma GCC diagnostic ignored "-Wformat-nonliteral"
-#endif
-#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_SECURITY
-#pragma GCC diagnostic ignored "-Wformat-security"
-#endif
+	/* upsdebug_INSTCMD_STARTING(item->info_type, value); */
 
 	if (!strcasecmp(item->info_type, "shutdown.return")) {
 
@@ -264,7 +259,7 @@ int	blazer_process_command(item_t *item, char *value, const size_t valuelen)
 		 * (thus the default of ondelay=3 mins) */
 
 		long	offdelay = strtol(dstate_getinfo("ups.delay.shutdown"), NULL, 10),
-				ondelay  = strtol(dstate_getinfo("ups.delay.start"), NULL, 10) / 60;
+			ondelay  = strtol(dstate_getinfo("ups.delay.start"), NULL, 10) / 60;
 		char	buf[SMALLBUF] = "";
 
 		if (ondelay <= 0) {
@@ -303,7 +298,7 @@ int	blazer_process_command(item_t *item, char *value, const size_t valuelen)
 
 		}
 
-		snprintf(value, valuelen, item->command, buf);
+		snprintf_dynamic(value, valuelen, item->command, "%s", buf);
 
 	} else if (!strcasecmp(item->info_type, "shutdown.stayoff")) {
 
@@ -326,7 +321,7 @@ int	blazer_process_command(item_t *item, char *value, const size_t valuelen)
 			snprintf(buf, sizeof(buf), "%02ld", offdelay / 60);
 		}
 
-		snprintf(value, valuelen, item->command, buf);
+		snprintf_dynamic(value, valuelen, item->command, "%s", buf);
 
 	} else if (!strcasecmp(item->info_type, "test.battery.start")) {
 
@@ -339,19 +334,34 @@ int	blazer_process_command(item_t *item, char *value, const size_t valuelen)
 
 		delay = delay / 60;
 
-		snprintf(value, valuelen, item->command, delay);
+		/* In various mapping tables, "%02d" is prevalent; actual
+		 * value is range-checked above to fit into a typical int
+		 */
+		if (validate_formatting_string(item->command, "%d", NUT_DYNAMICFORMATTING_DEBUG_LEVEL_SILENT) >= 0) {
+			/* The most likely case, should not cause much overhead */
+			snprintf_dynamic(value, valuelen, item->command, "%d", (int)delay);
+		} else {
+			if (validate_formatting_string(item->command, "", NUT_DYNAMICFORMATTING_DEBUG_LEVEL_SILENT) >= 0) {
+				/* A few mappings seem to just request the test
+				 * without parameters, so the second check is
+				 * for that eventuality
+				 */
+				snprintf(value, valuelen, "%s", item->command);
+			} else {
+				/* Finally try the actual long int (complaining
+				 * with default verbosity==1 if a bad fit) */
+				snprintf_dynamic(value, valuelen, item->command, "%ld", delay);
+			}
+		}
 
 	} else {
 
-		/* Don't know what happened */
+		/* Don't know what happened: unknown entry for pre-processing? */
+		/* upslog_INSTCMD_UNKNOWN(item->info_type, value); */
 		return -1;
 
 	}
 
-#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
-#pragma GCC diagnostic pop
-#endif
-
 	return 0;
 }
 
