@@ -26,7 +26,7 @@
 #include <stddef.h>
 #include "nut_stdint.h"
 
-#define MASTERGUARD_VERSION "Masterguard 0.02"
+#define MASTERGUARD_VERSION "Masterguard 0.03"
 
 /* series (for un-SKIP) */
 static char masterguard_my_series = '?';
@@ -600,31 +600,21 @@ static int masterguard_setvar(item_t *item, char *value, const size_t valuelen)
 		upsdebugx(2, "setvar: unknown dfl %s", item->dfl);
 		return -1;
 	}
-#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
-#pragma GCC diagnostic push
-#endif
-#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
-#pragma GCC diagnostic ignored "-Wformat-nonliteral"
-#endif
-#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_SECURITY
-#pragma GCC diagnostic ignored "-Wformat-security"
-#endif
+
 	switch (t) {
 		case 'd':
-			snprintf(value, valuelen, item->command, i);
+			snprintf_dynamic(value, valuelen, item->command, "%ld", i);
 			break;
 		case 'f':
-			snprintf(value, valuelen, item->command, f);
+			snprintf_dynamic(value, valuelen, item->command, "%f", f);
 			break;
 		case 's':
-			snprintf(value, valuelen, item->command, s);
+			snprintf_dynamic(value, valuelen, item->command, "%s", s);
 			break;
 		default:
 			break;
 	}
-#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
-#pragma GCC diagnostic pop
-#endif
+
 	return 0;
 ill:
 	upsdebugx(2, "setvar: illegal %s value %s", item->dfl, value);
