@@ -28,7 +28,7 @@
 #include "nut_stdint.h"
 
 #define DRIVER_NAME	"Opti-UPS driver"
-#define DRIVER_VERSION	"1.07"
+#define DRIVER_VERSION	"1.08"
 
 /* driver description structure */
 upsdrv_info_t upsdrv_info = {
@@ -234,8 +234,13 @@ static void optifill( ezfill_t *a, size_t len )
 /* Handle custom (but standardized) NUT commands */
 static int instcmd(const char *cmdname, const char *extra)
 {
+	/* May be used in logging below, but not as a command argument */
+	NUT_UNUSED_VARIABLE(extra);
+	upsdebug_INSTCMD_STARTING(cmdname, extra);
+
 	if (!strcasecmp(cmdname, "test.failure.start"))
 	{
+		upslog_INSTCMD_POWERSTATE_MAYBE(cmdname, extra);
 		optiquery( "Ts" );
 		return STAT_INSTCMD_HANDLED;
 	}
@@ -244,6 +249,7 @@ static int instcmd(const char *cmdname, const char *extra)
 		/* You do realize this will kill power to ourself.
 		 * Would probably only be useful for killing power for
 		 * a computer with upsmon in "secondary" mode */
+		upslog_INSTCMD_POWERSTATE_CHANGE(cmdname, extra);
 		if ( optimodel == OPTIMODEL_ZINTO )
 		{
 			optiquery( "Ct1" );
@@ -257,6 +263,7 @@ static int instcmd(const char *cmdname, const char *extra)
 	}
 	else if (!strcasecmp(cmdname, "load.on"))
 	{
+		upslog_INSTCMD_POWERSTATE_MAYBE(cmdname, extra);
 		if ( optimodel == OPTIMODEL_ZINTO )
 		{
 			optiquery( "Ct1" );
@@ -272,6 +279,7 @@ static int instcmd(const char *cmdname, const char *extra)
 	{
 		/* This shuts down the UPS.  When the power returns to the UPS,
 		 *   it will power back up in its default state. */
+		upslog_INSTCMD_POWERSTATE_CHANGE(cmdname, extra);
 		if ( optimodel == OPTIMODEL_ZINTO )
 		{
 			optiquery( "Ct1" );
@@ -288,6 +296,7 @@ static int instcmd(const char *cmdname, const char *extra)
 		/* This actually stays off as long as the batteries hold,
 		 *   if the line power comes back before the batteries die,
 		 *   the UPS will never powerup its output stage!!! */
+		upslog_INSTCMD_POWERSTATE_CHANGE(cmdname, extra);
 		if ( optimodel == OPTIMODEL_ZINTO )
 		{
 			optiquery( "Ct1" );
@@ -301,11 +310,12 @@ static int instcmd(const char *cmdname, const char *extra)
 	else if (!strcasecmp(cmdname, "shutdown.stop"))
 	{
 		/* Aborts a shutdown that is counting down via the Cs command */
+		upslog_INSTCMD_POWERSTATE_MAYBE(cmdname, extra);
 		optiquery( "Cs-0000001" );
 		return STAT_INSTCMD_HANDLED;
 	}
 
-	upslogx(LOG_NOTICE, "instcmd: unknown command [%s] [%s]", cmdname, extra);
+	upslog_INSTCMD_UNKNOWN(cmdname, extra);
 	return STAT_INSTCMD_UNKNOWN;
 }
 
@@ -314,6 +324,8 @@ static int setvar(const char *varname, const char *val)
 {
 	int status;
 
+	upsdebug_SET_STARTING(varname, val);
+
 	if (sscanf(val, "%d", &status) != 1) {
 		return STAT_SET_UNKNOWN;
 	}
@@ -326,6 +338,7 @@ static int setvar(const char *varname, const char *val)
 		return STAT_SET_HANDLED;
 	}
 
+	upslog_SET_UNKNOWN(varname, val);
 	return STAT_SET_UNKNOWN;
 }
 
