@@ -36,7 +36,7 @@
 
 #include <ctype.h>
 
-#define POWERPANEL_TEXT_VERSION	"Powerpanel-Text 0.62"
+#define POWERPANEL_TEXT_VERSION	"Powerpanel-Text 0.64"
 
 typedef struct {
 	float          i_volt;
@@ -146,40 +146,48 @@ static int powpan_instcmd(const char *cmdname, const char *extra)
 		return powpan_instcmd("beeper.enable", NULL);
 	}
 
+	/* May be used in logging below, but not as a command argument */
+	NUT_UNUSED_VARIABLE(extra);
+	upsdebug_INSTCMD_STARTING(cmdname, extra);
+
 	for (i = 0; cmdtab[i].cmd != NULL; i++) {
 
 		if (strcasecmp(cmdname, cmdtab[i].cmd)) {
 			continue;
 		}
 
+		upslog_INSTCMD_POWERSTATE_CHECKED(cmdname, extra);
 		if ((powpan_command(cmdtab[i].command) == 2) && (!strcasecmp(powpan_answer, "#0"))) {
 			return STAT_INSTCMD_HANDLED;
 		}
 
-		upslogx(LOG_ERR, "%s: command [%s] [%s] failed", __func__, cmdname, extra);
+		upslog_INSTCMD_FAILED(cmdname, extra);
 		return STAT_INSTCMD_FAILED;
 	}
 
 	if (!strcasecmp(cmdname, "shutdown.return")) {
+		upslog_INSTCMD_POWERSTATE_CHANGE(cmdname, extra);
 		if (offdelay < 60) {
 			snprintf(command, sizeof(command), "Z.%ld\r", offdelay / 6);
 		} else {
 			snprintf(command, sizeof(command), "Z%02ld\r", offdelay / 60);
 		}
 	} else if (!strcasecmp(cmdname, "shutdown.stayoff")) {
+		upslog_INSTCMD_POWERSTATE_CHANGE(cmdname, extra);
 		if (offdelay < 60) {
 			snprintf(command, sizeof(command), "S.%ld\r", offdelay / 6);
 		} else {
 			snprintf(command, sizeof(command), "S%02ld\r", offdelay / 60);
 		}
 	} else if (!strcasecmp(cmdname, "shutdown.reboot")) {
+		upslog_INSTCMD_POWERSTATE_CHANGE(cmdname, extra);
 		if (offdelay < 60) {
 			snprintf(command, sizeof(command), "S.%ldR%04ld\r", offdelay / 6, ondelay);
 		} else {
 			snprintf(command, sizeof(command), "S%02ldR%04ld\r", offdelay / 60, ondelay);
 		}
 	} else {
-		upslogx(LOG_NOTICE, "%s: command [%s] [%s] unknown", __func__, cmdname, extra);
+		upslog_INSTCMD_UNKNOWN(cmdname, extra);
 		return STAT_INSTCMD_UNKNOWN;
 	}
 
@@ -187,7 +195,7 @@ static int powpan_instcmd(const char *cmdname, const char *extra)
 		return STAT_INSTCMD_HANDLED;
 	}
 
-	upslogx(LOG_ERR, "%s: command [%s] [%s] failed", __func__, cmdname, extra);
+	upslog_INSTCMD_FAILED(cmdname, extra);
 	return STAT_INSTCMD_FAILED;
 }
 
@@ -196,6 +204,8 @@ static int powpan_setvar(const char *varname, const char *val)
 	char	command[SMALLBUF];
 	int 	i;
 
+	upsdebug_SET_STARTING(varname, val);
+
 	for (i = 0;  vartab[i].var != NULL; i++) {
 
 		if (strcasecmp(varname, vartab[i].var)) {
@@ -207,30 +217,18 @@ static int powpan_setvar(const char *varname, const char *val)
 			return STAT_SET_HANDLED;
 		}
 
-#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
-#pragma GCC diagnostic push
-#endif
-#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
-#pragma GCC diagnostic ignored "-Wformat-nonliteral"
-#endif
-#ifdef HAVE_PRAGMA_GCC_DIAGNOSTIC_IGNORED_FORMAT_SECURITY
-#pragma GCC diagnostic ignored "-Wformat-security"
-#endif
-		snprintf(command, sizeof(command), vartab[i].set, atoi(val));
-#ifdef HAVE_PRAGMAS_FOR_GCC_DIAGNOSTIC_IGNORED_FORMAT_NONLITERAL
-#pragma GCC diagnostic pop
-#endif
+		snprintf_dynamic(command, sizeof(command), vartab[i].set, "%d", atoi(val));
 
 		if ((powpan_command(command) == 2) && (!strcasecmp(powpan_answer, "#0"))) {
 			dstate_setinfo(varname, "%s", val);
 			return STAT_SET_HANDLED;
 		}
 
-		upslogx(LOG_ERR, "%s: setting variable [%s] to [%s] failed", __func__, varname, val);
-		return STAT_SET_UNKNOWN;
+		upslog_SET_FAILED(varname, val);
+		return STAT_SET_UNKNOWN;	/* FIXME: ..._FAILED? */
 	}
 
-	upslogx(LOG_ERR, "%s: variable [%s] not found", __func__, varname);
+	upslog_SET_UNKNOWN(varname, val);
 	return STAT_SET_UNKNOWN;
 }
 
