@@ -57,7 +57,13 @@
 #ifndef NUT_USB_COMMON_H
 #define NUT_USB_COMMON_H
 
-#include "config.h"	/* be sure to know all about the system config */
+/* "config.h" is generated by autotools and lacks a header guard, so
+ * we use an unambiguously named macro we know we must have, as one.
+ * It must be the first header: be sure to know all about system config.
+ */
+#ifndef NUT_NETVERSION
+# include "config.h"
+#endif
 
 /* Note: usb-common.h (this file) is included by nut_libusb.h,
  * so not looping the includes ;)
@@ -107,75 +113,79 @@
  * any use-case.
  */
  typedef uint8_t usb_ctrl_requesttype;
- #define USB_CTRL_REQUESTTYPE_MIN	0
- #define USB_CTRL_REQUESTTYPE_MAX	UINT8_MAX
+# define USB_CTRL_REQUESTTYPE_MIN	0
+# define USB_CTRL_REQUESTTYPE_MAX	UINT8_MAX
 
  typedef uint8_t usb_ctrl_request;
- #define USB_CTRL_REQUEST_MIN	0
- #define USB_CTRL_REQUEST_MAX	UINT8_MAX
+# define USB_CTRL_REQUEST_MIN	0
+# define USB_CTRL_REQUEST_MAX	UINT8_MAX
 
  typedef unsigned char usb_ctrl_endpoint;
- #define USB_CTRL_ENDPOINT_MIN	0
- #define USB_CTRL_ENDPOINT_MAX	UCHAR_MAX
+# define USB_CTRL_ENDPOINT_MIN	0
+# define USB_CTRL_ENDPOINT_MAX	UCHAR_MAX
 
  typedef uint16_t usb_ctrl_msgvalue;
- #define USB_CTRL_MSGVALUE_MIN	0
- #define USB_CTRL_MSGVALUE_MAX	UINT16_MAX
+# define USB_CTRL_MSGVALUE_MIN	0
+# define USB_CTRL_MSGVALUE_MAX	UINT16_MAX
+
+ typedef uint8_t usb_ctrl_cfgindex;
+# define USB_CTRL_CFGINDEX_MIN	0
+# define USB_CTRL_CFGINDEX_MAX	UINT8_MAX
 
  typedef uint16_t usb_ctrl_repindex;
- #define USB_CTRL_REPINDEX_MIN	0
- #define USB_CTRL_REPINDEX_MAX	UINT16_MAX
+# define USB_CTRL_REPINDEX_MIN	0
+# define USB_CTRL_REPINDEX_MAX	UINT16_MAX
 
  typedef uint8_t usb_ctrl_strindex;
- #define USB_CTRL_STRINDEX_MIN	0
- #define USB_CTRL_STRINDEX_MAX	UINT8_MAX
+# define USB_CTRL_STRINDEX_MIN	0
+# define USB_CTRL_STRINDEX_MAX	UINT8_MAX
 
  typedef uint8_t usb_ctrl_descindex;
- #define USB_CTRL_DESCINDEX_MIN	0
- #define USB_CTRL_DESCINDEX_MAX	UINT8_MAX
+# define USB_CTRL_DESCINDEX_MIN	0
+# define USB_CTRL_DESCINDEX_MAX	UINT8_MAX
 
  typedef unsigned char* usb_ctrl_charbuf;
  typedef unsigned char usb_ctrl_char;
- #define USB_CTRL_CHAR_MIN	0
- #define USB_CTRL_CHAR_MAX	UCHAR_MAX
+# define USB_CTRL_CHAR_MIN	0
+# define USB_CTRL_CHAR_MAX	UCHAR_MAX
 
  /* Here MIN/MAX should not matter much, type mostly used for casting */
  typedef uint16_t usb_ctrl_charbufsize;
- #define USB_CTRL_CHARBUFSIZE_MIN	0
- #define USB_CTRL_CHARBUFSIZE_MAX	UINT16_MAX
- #define PRI_NUT_USB_CTRL_CHARBUFSIZE PRIu16
+# define USB_CTRL_CHARBUFSIZE_MIN	0
+# define USB_CTRL_CHARBUFSIZE_MAX	UINT16_MAX
+# define PRI_NUT_USB_CTRL_CHARBUFSIZE PRIu16
 
  typedef unsigned int usb_ctrl_timeout_msec;	/* in milliseconds */
 	/* Note: there does not seem to be a standard type
 	 * for milliseconds, like there is an useconds_t */
- #define USB_CTRL_TIMEOUTMSEC_MIN	0
- #define USB_CTRL_TIMEOUTMSEC_MAX	UINT_MAX
+# define USB_CTRL_TIMEOUTMSEC_MIN	0
+# define USB_CTRL_TIMEOUTMSEC_MAX	UINT_MAX
 
  /* defines */
- #define USB_CLASS_PER_INTERFACE	LIBUSB_CLASS_PER_INTERFACE
- #define USB_DT_STRING				LIBUSB_DT_STRING
- #define USB_ENDPOINT_IN			LIBUSB_ENDPOINT_IN
- #define USB_ENDPOINT_OUT			LIBUSB_ENDPOINT_OUT
- #define USB_RECIP_ENDPOINT			LIBUSB_RECIPIENT_ENDPOINT
- #define USB_RECIP_INTERFACE		LIBUSB_RECIPIENT_INTERFACE
- #define USB_REQ_SET_DESCRIPTOR		LIBUSB_REQUEST_SET_DESCRIPTOR
- #define USB_TYPE_CLASS				LIBUSB_REQUEST_TYPE_CLASS
- #define USB_TYPE_VENDOR			LIBUSB_REQUEST_TYPE_VENDOR
+# define USB_CLASS_PER_INTERFACE	LIBUSB_CLASS_PER_INTERFACE
+# define USB_DT_STRING				LIBUSB_DT_STRING
+# define USB_ENDPOINT_IN			LIBUSB_ENDPOINT_IN
+# define USB_ENDPOINT_OUT			LIBUSB_ENDPOINT_OUT
+# define USB_RECIP_ENDPOINT			LIBUSB_RECIPIENT_ENDPOINT
+# define USB_RECIP_INTERFACE		LIBUSB_RECIPIENT_INTERFACE
+# define USB_REQ_SET_DESCRIPTOR		LIBUSB_REQUEST_SET_DESCRIPTOR
+# define USB_TYPE_CLASS				LIBUSB_REQUEST_TYPE_CLASS
+# define USB_TYPE_VENDOR			LIBUSB_REQUEST_TYPE_VENDOR
 
 /* Codebase updated to use LIBUSB_* tokens:
- #define ERROR_ACCESS		LIBUSB_ERROR_ACCESS
- #define ERROR_BUSY			LIBUSB_ERROR_BUSY
- #define ERROR_IO			LIBUSB_ERROR_IO
- #define ERROR_NO_DEVICE	LIBUSB_ERROR_NO_DEVICE
- #define ERROR_NOT_FOUND	LIBUSB_ERROR_NOT_FOUND
- #define ERROR_OVERFLOW		LIBUSB_ERROR_OVERFLOW
- #define ERROR_PIPE			LIBUSB_ERROR_PIPE
- #define ERROR_TIMEOUT		LIBUSB_ERROR_TIMEOUT
- #define ERROR_NO_MEM   LIBUSB_ERROR_NO_MEM
- #define ERROR_INVALID_PARAM  LIBUSB_ERROR_INVALID_PARAM
- #define ERROR_INTERRUPTED    LIBUSB_ERROR_INTERRUPTED
- #define ERROR_NOT_SUPPORTED  LIBUSB_ERROR_NOT_SUPPORTED
- #define ERROR_OTHER          LIBUSB_ERROR_OTHER
+# define ERROR_ACCESS		LIBUSB_ERROR_ACCESS
+# define ERROR_BUSY			LIBUSB_ERROR_BUSY
+# define ERROR_IO			LIBUSB_ERROR_IO
+# define ERROR_NO_DEVICE	LIBUSB_ERROR_NO_DEVICE
+# define ERROR_NOT_FOUND	LIBUSB_ERROR_NOT_FOUND
+# define ERROR_OVERFLOW		LIBUSB_ERROR_OVERFLOW
+# define ERROR_PIPE			LIBUSB_ERROR_PIPE
+# define ERROR_TIMEOUT		LIBUSB_ERROR_TIMEOUT
+# define ERROR_NO_MEM   LIBUSB_ERROR_NO_MEM
+# define ERROR_INVALID_PARAM  LIBUSB_ERROR_INVALID_PARAM
+# define ERROR_INTERRUPTED    LIBUSB_ERROR_INTERRUPTED
+# define ERROR_NOT_SUPPORTED  LIBUSB_ERROR_NOT_SUPPORTED
+# define ERROR_OTHER          LIBUSB_ERROR_OTHER
 */
 
  /* Functions, including range-checks to convert data types of the two APIs.
@@ -363,16 +373,16 @@
 #endif
 
  /* Functions for which simple mappings seem to suffice (no build warnings emitted): */
- #define usb_claim_interface	libusb_claim_interface
- #define usb_clear_halt			libusb_clear_halt
- #define usb_close				libusb_close
- #define usb_set_configuration	libusb_set_configuration
- #define usb_release_interface	libusb_release_interface
- #define usb_reset				libusb_reset_device
+# define usb_claim_interface	libusb_claim_interface
+# define usb_clear_halt			libusb_clear_halt
+# define usb_close				libusb_close
+# define usb_set_configuration	libusb_set_configuration
+# define usb_release_interface	libusb_release_interface
+# define usb_reset				libusb_reset_device
 
  /* FIXME: some original libusb1.c code cast the (int) argument
   * as (enum libusb_error) - should we force that in the macro? */
- #define nut_usb_strerror(a)	libusb_strerror(a)
+# define nut_usb_strerror(a)	libusb_strerror(a)
 #endif  /* WITH_LIBUSB_1_0 */
 
 /* Note: Checked above that in practice we handle some one libusb API */
@@ -396,66 +406,70 @@
  /* no typedef for usb_dev_handle - part of libusb-0.1 API names */
 
  typedef int usb_ctrl_requesttype;
- #define USB_CTRL_REQUESTTYPE_MIN	INT_MIN
- #define USB_CTRL_REQUESTTYPE_MAX	INT_MAX
+# define USB_CTRL_REQUESTTYPE_MIN	INT_MIN
+# define USB_CTRL_REQUESTTYPE_MAX	INT_MAX
 
  typedef int usb_ctrl_request;
- #define USB_CTRL_REQUEST_MIN	INT_MIN
- #define USB_CTRL_REQUEST_MAX	INT_MAX
+# define USB_CTRL_REQUEST_MIN	INT_MIN
+# define USB_CTRL_REQUEST_MAX	INT_MAX
 
  typedef int usb_ctrl_endpoint;
- #define USB_CTRL_ENDPOINT_MIN	INT_MIN
- #define USB_CTRL_ENDPOINT_MAX	INT_MAX
+# define USB_CTRL_ENDPOINT_MIN	INT_MIN
+# define USB_CTRL_ENDPOINT_MAX	INT_MAX
 
  typedef int usb_ctrl_msgvalue;
- #define USB_CTRL_MSGVALUE_MIN	INT_MIN
- #define USB_CTRL_MSGVALUE_MAX	INT_MAX
+# define USB_CTRL_MSGVALUE_MIN	INT_MIN
+# define USB_CTRL_MSGVALUE_MAX	INT_MAX
+
+ typedef uint8_t usb_ctrl_cfgindex;
+# define USB_CTRL_CFGINDEX_MIN	0
+# define USB_CTRL_CFGINDEX_MAX	UINT8_MAX
 
  typedef int usb_ctrl_repindex;
- #define USB_CTRL_REPINDEX_MIN	INT_MIN
- #define USB_CTRL_REPINDEX_MAX	INT_MAX
+# define USB_CTRL_REPINDEX_MIN	INT_MIN
+# define USB_CTRL_REPINDEX_MAX	INT_MAX
 
  typedef int usb_ctrl_strindex;
- #define USB_CTRL_STRINDEX_MIN	INT_MIN
- #define USB_CTRL_STRINDEX_MAX	INT_MAX
+# define USB_CTRL_STRINDEX_MIN	INT_MIN
+# define USB_CTRL_STRINDEX_MAX	INT_MAX
 
  typedef int usb_ctrl_descindex;
- #define USB_CTRL_DESCINDEX_MIN	INT_MIN
- #define USB_CTRL_DESCINDEX_MAX	INT_MAX
+# define USB_CTRL_DESCINDEX_MIN	INT_MIN
+# define USB_CTRL_DESCINDEX_MAX	INT_MAX
 
  /* Here MIN/MAX should not matter much, type mostly used for casting */
  typedef char* usb_ctrl_charbuf;
  typedef char usb_ctrl_char;
- #define USB_CTRL_CHAR_MIN	CHAR_MIN
- #define USB_CTRL_CHAR_MAX	CHAR_MAX
+# define USB_CTRL_CHAR_MIN	CHAR_MIN
+# define USB_CTRL_CHAR_MAX	CHAR_MAX
 
  typedef int usb_ctrl_charbufsize;
- #define USB_CTRL_CHARBUFSIZE_MIN	INT_MIN
- #define USB_CTRL_CHARBUFSIZE_MAX	INT_MAX
+# define USB_CTRL_CHARBUFSIZE_MIN	INT_MIN
+# define USB_CTRL_CHARBUFSIZE_MAX	INT_MAX
  /* There is no PRIi :) So we define directly by spec */
- #define PRI_NUT_USB_CTRL_CHARBUFSIZE "i"
+# define PRI_NUT_USB_CTRL_CHARBUFSIZE "i"
 
  typedef int usb_ctrl_timeout_msec;	/* in milliseconds */
- #define USB_CTRL_TIMEOUTMSEC_MIN	INT_MIN
- #define USB_CTRL_TIMEOUTMSEC_MAX	INT_MAX
+# define USB_CTRL_TIMEOUTMSEC_MIN	INT_MIN
+# define USB_CTRL_TIMEOUTMSEC_MAX	INT_MAX
 
  /* defines */
- #define LIBUSB_ERROR_ACCESS		-EACCES
- #define LIBUSB_ERROR_BUSY			-EBUSY
- #define LIBUSB_ERROR_IO			-EIO
- #define LIBUSB_ERROR_NO_DEVICE	-ENODEV
- #define LIBUSB_ERROR_NOT_FOUND	-ENOENT
- #define LIBUSB_ERROR_OVERFLOW		-EOVERFLOW
- #define LIBUSB_ERROR_PIPE			-EPIPE
- #define LIBUSB_ERROR_TIMEOUT		-ETIMEDOUT
- #define LIBUSB_ERROR_NO_MEM		-ENOMEM
- #define LIBUSB_ERROR_INVALID_PARAM		-EINVAL
- #define LIBUSB_ERROR_INTERRUPTED		-EINTR
- #define LIBUSB_ERROR_NOT_SUPPORTED	-ENOSYS
- #define LIBUSB_ERROR_OTHER		-ERANGE
+# define LIBUSB_ERROR_ACCESS		-EACCES
+# define LIBUSB_ERROR_BUSY			-EBUSY
+# define LIBUSB_ERROR_IO			-EIO
+# define LIBUSB_ERROR_NO_DEVICE	-ENODEV
+# define LIBUSB_ERROR_NOT_FOUND	-ENOENT
+# define LIBUSB_ERROR_OVERFLOW		-EOVERFLOW
+# define LIBUSB_ERROR_PIPE			-EPIPE
+# define LIBUSB_ERROR_TIMEOUT		-ETIMEDOUT
+# define LIBUSB_ERROR_NO_MEM		-ENOMEM
+# define LIBUSB_ERROR_INVALID_PARAM		-EINVAL
+# define LIBUSB_ERROR_INTERRUPTED		-EINTR
+# define LIBUSB_ERROR_NOT_SUPPORTED	-ENOSYS
+# define LIBUSB_ERROR_OTHER		-ERANGE
 
  /* Functions for which simple mappings seem to suffice (no build warnings emitted): */
- #define nut_usb_strerror(a)	usb_strerror()
+# define nut_usb_strerror(a)	usb_strerror()
 #endif  /* WITH_LIBUSB_0_1 */
 
 /* USB standard timeout [ms] */
@@ -546,4 +560,9 @@ void nut_usb_addvars(void);
  * here, to use in several USB-capable drivers. */
 void warn_if_bad_usb_port_filename(const char *fn);
 
+/* Retrieves string descriptor from device, decoded to 8-bit string,
+ * applies retries on failures, and assumes en_US in case the
+ * langid descriptor is invalid. */
+int nut_usb_get_string(usb_dev_handle *udev, int StringIdx, char *buf, size_t buflen);
+
 #endif /* NUT_USB_COMMON_H */
