@@ -1,6 +1,11 @@
 /* user.c - supporting elements of user handling functions for upsd
 
    Copyright (C) 2001  Russell Kroll <rkroll@exploits.org>
+	2005	Arnaud Quette <arnaud.quette@free.fr>
+	2007	Peter Selinger <selinger@users.sourceforge.net>
+	2008	Arjen de Korte <adkorte-guest@alioth.debian.org>
+	2013	Emilien Kia <kiae.dev@gmail.com>
+	2020	Jim Klimov <jimklimov@gmail.com>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,6 +22,9 @@
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
 
+#ifndef NUT_USER_H_SEEN
+#define NUT_USER_H_SEEN 1
+
 #ifdef __cplusplus
 /* *INDENT-OFF* */
 extern "C" {
@@ -39,3 +47,4 @@ void check_perms(const char *fn);
 /* *INDENT-ON* */
 #endif
 
+#endif /* NUT_USER_H_SEEN */
