@@ -514,7 +514,7 @@ static void handle_arg_cidr(const char *arg_addr, int *auto_nets_ptr)
 			if (FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
 				FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS, 
 				NULL, dwRetVal, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),   
-				// Default language
+				/* Default language */
 				(LPTSTR) & lpMsgBuf, 0, NULL) && lpMsgBuf
 			) {
 				fatalx(EXIT_FAILURE, "%s: %s",
@@ -1257,7 +1257,7 @@ int main(int argc, char *argv[])
 
 		switch(opt_ret) {
 			case 't':
-				{ // scoping
+				{ /* scoping */
 					long	l;
 					char	*s = NULL;
 					int	errno_saved;
